#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = require("./util");
// import { localCloudfront } from './cloudfront'
// Directory and utilities
const dir = process.cwd();
const configFile = path_1.default.join(dir, 'lambda-aws.json');
// Utilities for reading configuration
const hasConfig = () => fs_1.default.existsSync(dir) && fs_1.default.existsSync(configFile);
const readConfig = () => JSON.parse(fs_1.default.readFileSync(configFile, 'utf8'));
const writeConfig = () => fs_1.default.writeFileSync(configFile, JSON.stringify(config, null, 4), 'utf8');
const defaultConfig = () => ({
    name: path_1.default.basename(dir),
    prefix: { 'dev': 'dev-', 'prod': 'prod-' },
    domain: '',
    region: [],
    service: []
});
// Configuration object
const config = hasConfig() ? readConfig() : defaultConfig();
/**
 * INITIALIZATION
 * > laws init
 *
 * Create a configuration JSON file in this directory.
 */
function init() {
    if (hasConfig())
        throw new Error('already initialized');
    console.log('Writing configuration');
    writeConfig();
}
/**
 * ADD SERVICE
 * > laws add api/cloudfront
 */
function add(service) {
    if (!(0, util_1.isServiceId)(service))
        throw new Error('invalid service type');
    config.service.push({
        id: service,
        type: service,
        handler: 'dist/' + service + '.handler'
    });
    writeConfig();
}
function local(service) {
    switch (service) {
        case 'cloudfront': return localServer();
    }
}
function localServer() {
}
function logHelp() {
    console.log('laws init');
}
if (process.argv.length > 2) {
    const command = process.argv[2];
    try {
        switch (command) {
            case 'init':
                init();
                break;
            case 'add':
                add(process.argv[3]);
                break;
            case 'local':
                local(process.argv[3]);
                break;
        }
    }
    catch (error) {
        console.log(error.toString());
    }
}
else
    logHelp();
