"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudFrontDistribution = void 0;
const client_cloudfront_1 = require("@aws-sdk/client-cloudfront");
const constant_1 = require("./constant");
class CloudFrontDistribution {
    static list() {
        return __awaiter(this, void 0, void 0, function* () {
            const client = new client_cloudfront_1.CloudFrontClient({});
            const items = [];
            // Page through all distributions
            let next;
            do {
                const result = yield client.send(new client_cloudfront_1.ListDistributionsCommand({
                    Marker: next
                }));
                const distributions = result.DistributionList;
                if (distributions && distributions.Items && distributions.Items.length > 0) {
                    items.push(...distributions.Items.map((item) => {
                        return new CloudFrontDistribution();
                    }));
                    if (distributions.IsTruncated)
                        next = distributions.NextMarker;
                }
                else
                    break;
            } while (next != null);
            return items;
        });
    }
    constructor(config, service) {
        this.origin = [];
        this.alias = [];
        this.client = new client_cloudfront_1.CloudFrontClient({ region: constant_1.CLOUDFRONT_REGION });
        this.config = config;
        this.service = service || { type: 'cloudfront' };
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            const origins = yield this.getOrigins();
            const result = yield this.client.send(new client_cloudfront_1.CreateDistributionCommand({
                DistributionConfig: {
                    Enabled: true,
                    Origins: origins,
                    Aliases: this.getAliases(),
                    ViewerCertificate: this.certificate.getViewerCertificate(),
                    DefaultCacheBehavior: this.getCacheBehavior(origins),
                    Comment: this.getComment(),
                    CallerReference: '',
                    PriceClass: 'PriceClass_All'
                }
            }));
            return this.setDistribution(result.Distribution, result.ETag);
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.send(new client_cloudfront_1.UpdateDistributionCommand({
                Id: this.getId(),
                IfMatch: this.getETag(),
                DistributionConfig: this.configuration
            }));
            return this.setDistribution(result.Distribution, result.ETag);
        });
    }
    exists() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.get();
                return true;
            }
            catch (error) {
                return false;
            }
        });
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.send(new client_cloudfront_1.GetDistributionCommand({
                Id: this.id
            }));
            this.setDistribution(result.Distribution, result.ETag);
            return this;
        });
    }
    getTags() {
        return __awaiter(this, void 0, void 0, function* () {
            this.client.send(new client_cloudfront_1.ListTagsForResourceCommand({
                Resource: this.getArn()
            }));
        });
    }
    getOrigins() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                Items: [
                // {
                // 	DomainName: cdnDomain + '.s3.amazonaws.com',
                // 	Id: 'S3-' + APP_NAME,
                // 	S3OriginConfig: {
                // 		OriginAccessIdentity: 'origin-access-identity/cloudfront/' + accessId
                // 	}
                // }
                ],
                Quantity: 1
            };
        });
    }
    getComputeOrigins() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                Items: [],
                Quantity: 0
            };
        });
    }
    getTargetOriginId() {
        return '';
    }
    getAliases() {
        // If no domain configuration is set
        if (!this.config)
            return { Items: [], Quantity: 0 };
        // If there are multiple domain aliases
        if (Array.isArray(this.config.domain))
            return {
                Items: this.config.domain,
                Quantity: this.config.domain.length
            };
        // Single domain alias
        return {
            Items: [
                this.config.domain
            ],
            Quantity: 1
        };
    }
    getCacheBehavior(origins) {
        return this.getComputeCacheBehavior(origins);
    }
    getStorageCacheBehavior(origins) {
        return {
            TargetOriginId: origins.Items[0].Id,
            ViewerProtocolPolicy: 'redirect-to-https',
            AllowedMethods: {
                Items: [
                    'HEAD', 'GET', 'OPTIONS'
                ],
                CachedMethods: {
                    Items: ['HEAD', 'GET', 'OPTIONS'],
                    Quantity: 3
                },
                Quantity: 3
            },
            CachePolicyId: '658327ea-f89d-4fab-a63d-7e88639e58f6',
            OriginRequestPolicyId: '88a5eaf4-2fd4-4709-b370-b4c650ea3fcf'
        };
    }
    getComputeCacheBehavior(origins) {
        return {
            TargetOriginId: origins.Items[0].Id,
            CachePolicyId: '4135ea2d-6df8-44a3-9df3-4b5a84be39ad',
            ViewerProtocolPolicy: 'redirect-to-https',
            AllowedMethods: {
                Items: [
                    'GET', 'HEAD', 'POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE'
                ],
                Quantity: 7
            },
            SmoothStreaming: false
        };
    }
    getComment() {
        return '';
    }
    getDomainName() {
        if (!this.domainName)
            throw new Error('not loaded');
        return this.domainName;
    }
    getId() {
        return this.id;
    }
    getArn() {
        if (!this.arn)
            throw new Error('not loaded');
        return this.arn;
    }
    setDistribution(config, etag) {
        if (config) {
            this.arn = config.ARN;
            this.domainName = config.DomainName;
            this.configuration = config.DistributionConfig;
        }
        this.etag = etag;
        return this;
    }
    getETag() {
        return this.etag;
    }
}
exports.CloudFrontDistribution = CloudFrontDistribution;
