"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.proxyResultToHttpV2 = exports.httpToProxyEventV2 = exports.proxyServer = void 0;
const http_1 = __importDefault(require("http"));
const url_1 = __importDefault(require("url"));
const BODY_METHODS = new Set(['post', 'patch', 'put']);
function proxyServer(handler, port = 3000) {
    return __awaiter(this, void 0, void 0, function* () {
        const delimiter = handler.lastIndexOf('.');
        const handlerPath = handler.substring(0, delimiter);
        const handlerFunction = handler.substring(delimiter + 1);
        const server = http_1.default.createServer((req, res) => {
            httpToProxyEventV2(req)
                .then((event) => {
                return require(handlerPath)[handlerFunction](event);
            })
                .then((result) => {
                return proxyResultToHttpV2(result, res);
            })
                .then((proxied) => {
            })
                .catch((error) => {
                console.log('error', error);
                res.writeHead(500).end(error.toString());
            });
        });
        return new Promise((resolve) => {
            server.listen(port, () => {
                resolve(server);
            });
        });
    });
}
exports.proxyServer = proxyServer;
function httpToProxyEventV2(req) {
    return __awaiter(this, void 0, void 0, function* () {
        // Parse request method
        if (!req.method || !req.url)
            throw new Error('invalid request');
        const method = req.method.toLowerCase();
        // Get path from URL
        const target = url_1.default.parse(req.url);
        if (!target.pathname)
            throw new Error('invalid path');
        const rawPath = target.pathname;
        // Parse query string parameters
        const rawQueryString = target.query || '';
        const queryStringParameters = {};
        if (target.query)
            for (const [key, value] of new URLSearchParams(target.query))
                queryStringParameters[key] = Array.isArray(value) ? value[0] : value;
        // Parse headers
        const headers = {};
        Object.keys(req.headers).forEach((key) => {
            const value = req.headers[key];
            if (value)
                headers[key] = Array.isArray(value) ? value[0] : value;
        });
        // Parse body string
        let body = undefined;
        if (BODY_METHODS.has(this.method)) {
            body = yield new Promise((resolve) => {
                const chunks = [];
                this.req.on('data', (chunk) => {
                    chunks.push(chunk);
                });
                this.req.on('end', () => {
                    resolve(chunks.join(''));
                });
            });
        }
        // Construct proxy object
        return {
            version: '2.0',
            routeKey: '$default',
            rawPath,
            rawQueryString,
            cookies: [],
            headers,
            queryStringParameters,
            body,
            isBase64Encoded: false,
            stageVariables: {},
            requestContext: {
                accountId: '',
                apiId: '',
                routeKey: '$default',
                requestId: '',
                stage: '$default',
                time: '',
                timeEpoch: 12,
                http: {
                    method,
                    path: rawPath,
                    protocol: 'HTTP/1.1',
                    sourceIp: '',
                    userAgent: ''
                },
                domainName: '',
                domainPrefix: ''
            }
        };
    });
}
exports.httpToProxyEventV2 = httpToProxyEventV2;
function proxyResultToHttpV2(result, res) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof result === 'string') {
            res.writeHead(200);
            res.write(result);
            res.end();
        }
        else {
            res.writeHead(result.statusCode || 200);
            res.write(result.body);
            if (result.headers) {
                for (const key of Object.keys(result.headers)) {
                    const value = result.headers[key];
                    res.setHeader(key, value.toString());
                }
            }
            res.end();
        }
    });
}
exports.proxyResultToHttpV2 = proxyResultToHttpV2;
