"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Request = void 0;
const url_1 = __importDefault(require("url"));
const error_1 = require("./error");
const constant_1 = require("./constant");
/**
 * @class 		Request
 * @desc 		Represents an incoming HTTP request
 */
class Request {
    constructor(req) {
        this.query = {};
        // Set for HTTP requests
        this.header = {};
        this.req = req;
        // Parse request method
        if (!req.method || !req.url)
            throw new Error('invalid request');
        this.method = req.method.toLowerCase();
        const target = url_1.default.parse(req.url);
        if (!target.pathname)
            throw error_1.InvalidRequest;
        this.url = target.pathname;
        // Parse path name and query parameters
        if (target.query)
            for (const [key, value] of new URLSearchParams(target.query))
                this.query[key] = Array.isArray(value) ? value[0] : value;
        Object.keys(req.headers).forEach((key) => {
            const value = req.headers[key];
            if (value)
                this.header[key] = Array.isArray(value) ? value[0] : value;
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            // Parse request body into JSON object
            let data;
            if (constant_1.BODY_METHODS.has(this.method)) {
                data = yield new Promise((resolve) => {
                    const chunks = [];
                    this.req.on('data', (chunk) => {
                        chunks.push(chunk);
                    });
                    this.req.on('end', () => {
                        const data = chunks.join('');
                        try {
                            // TODO: urlencoded, XML, JSON - check content type
                            resolve(JSON.parse(data));
                        }
                        catch (error) {
                            resolve({});
                        }
                    });
                });
            }
        });
    }
    hasScope(scope) {
        // TODO: check request auth
        return true;
    }
    hasContentType(contentType) {
        // TODO: check headers
        return true;
    }
}
exports.Request = Request;
