"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainName = void 0;
const client_route_53_1 = require("@aws-sdk/client-route-53");
const _1 = require(".");
class DomainName {
    /**
     * @constructor
     * @param 	{string} id - the hosted zone ID
     * @param 	{string} name - domain name
     */
    constructor(id, name) {
        this.recordSize = 0;
        this.id = id;
        this.name = name;
    }
    /**
     * @func 	getSubdomain
     * @desc 	Get a representation of a subdomain of this domain name.
     * @param 	{string} subdomain - prepend to domain name
     */
    getSubdomain(subdomain) {
        return new DomainName(this.id, [subdomain.replace(/\.$/, ''), this.name].join('.'));
    }
    static find(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let next;
            while (true) {
                const result = yield DomainName.client.send(new client_route_53_1.ListHostedZonesCommand({
                    Marker: next
                }));
                // Add to domains
                if (result.HostedZones) {
                    const hostedZone = result.HostedZones.find((zone) => zone.Name && zone.Name.startsWith(name));
                    if (hostedZone && hostedZone.Id)
                        return new DomainName(hostedZone.Id, hostedZone.Name);
                }
                if (result.IsTruncated)
                    next = result.NextMarker;
                else
                    break;
            }
            return null;
        });
    }
    static list() {
        return __awaiter(this, void 0, void 0, function* () {
            const domains = [];
            let next;
            while (true) {
                const result = yield DomainName.client.send(new client_route_53_1.ListHostedZonesCommand({
                    Marker: next
                }));
                // Add to domains
                if (result.HostedZones)
                    domains.push(...result.HostedZones.map((zone) => new DomainName(zone.Id, zone.Name)));
                if (result.IsTruncated)
                    next = result.NextMarker;
                else
                    break;
            }
            return domains;
        });
    }
    exists() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.get();
                return true;
            }
            catch (error) {
                return false;
            }
        });
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield DomainName.client.send(new client_route_53_1.GetHostedZoneCommand({
                Id: this.id
            }));
            if (result.HostedZone) {
                this.name = result.HostedZone.Name;
                this.recordSize = result.HostedZone.ResourceRecordSetCount || 0;
            }
            // Store list of name servers
            if (result.DelegationSet)
                this.nameServers = result.DelegationSet.NameServers;
            return this;
        });
    }
    getRecords() {
        return __awaiter(this, void 0, void 0, function* () {
            const records = [];
            let next;
            while (true) {
                const result = yield DomainName.client.send(new client_route_53_1.ListResourceRecordSetsCommand({
                    HostedZoneId: this.id,
                    StartRecordIdentifier: next
                }));
                if (result.ResourceRecordSets)
                    records.push(...result.ResourceRecordSets.map((record) => new _1.DomainRecord(this)));
                if (result.IsTruncated)
                    next = result.NextRecordIdentifier;
                else
                    break;
            }
            return records;
        });
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO
            yield DomainName.client.send(new client_route_53_1.CreateHostedZoneCommand({
                Name: this.name,
                CallerReference: '',
                HostedZoneConfig: {
                    Comment: ''
                }
            }));
        });
    }
    archive() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO delete/archive resource records
            yield DomainName.client.send(new client_route_53_1.DeleteHostedZoneCommand({
                Id: this.id
            }));
        });
    }
    getId() {
        return this.id;
    }
    getName() {
        return this.name || '';
    }
    getNameServers() {
        return this.nameServers || [];
    }
}
exports.DomainName = DomainName;
DomainName.client = new client_route_53_1.Route53Client({});
