"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIGateway = void 0;
const client_apigatewayv2_1 = require("@aws-sdk/client-apigatewayv2");
const util_1 = require("../util");
class APIGateway {
    constructor(region, config, service) {
        this.client = new client_apigatewayv2_1.ApiGatewayV2Client({ region });
        this.config = config;
        this.service = service;
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.send(new client_apigatewayv2_1.CreateApiCommand({
                Name: this.getName(),
                ProtocolType: 'HTTP',
                ApiKeySelectionExpression: '$request.header.x-api-key',
                Description: this.getDescription(),
                RouteSelectionExpression: '$request.method $request.path'
            }));
            this.setApiId(result.ApiId);
        });
    }
    // async getIntegrations() {
    // 	const result = await this.region.getClient().send(new GetIntegrationsCommand({
    // 		ApiId: this.id
    // 	}))
    //
    // 	result.Items.forEach((item) => {
    //
    // 	})
    // }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.send(new client_apigatewayv2_1.GetApiCommand({
                ApiId: this.getApiId()
            }));
            this.setApiId(result.ApiId);
            // this.endpoint = result.ApiEndpoint
            return this;
        });
    }
    createAuthorizer() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.send(new client_apigatewayv2_1.CreateAuthorizerCommand({
                ApiId: this.getId(),
                Name: this.getName(),
                AuthorizerType: 'JWT',
                IdentitySource: ['$request.header.Authorization'],
                JwtConfiguration: {
                    Audience: []
                }
            }));
        });
    }
    exists() {
        return __awaiter(this, void 0, void 0, function* () {
            return false;
        });
    }
    getId() {
        return this.service.id || this.service.type;
    }
    getApiId() {
        if (!this.service.aws)
            throw new Error('not loaded/created');
        return this.service.aws.api_id;
    }
    setApiId(id) {
        if (!this.service.aws)
            this.service.aws = {};
        this.service.aws.api_id = id;
    }
    getName() {
        return (0, util_1.getServiceName)(this.config, this.getId());
    }
    getDescription() {
        return 'API integration';
    }
    getClient() {
        return this.client;
    }
}
exports.APIGateway = APIGateway;
