"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LambdaFunction = void 0;
const client_lambda_1 = require("@aws-sdk/client-lambda");
/**
 * @class 	ComputeFunction
 * @desc 	Represents a Compute function
 */
class LambdaFunction {
    constructor(region, name) {
        this.handler = 'dist/index.handler';
        this.memorySize = 256;
        this.region = region;
        this.name = name;
    }
    exists() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.functionExists != null)
                return this.functionExists;
            try {
                yield this.get();
                return true;
            }
            catch (error) {
                return false;
            }
        });
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.region.getClient().send(new client_lambda_1.GetFunctionCommand({
                FunctionName: this.getName()
            }));
            if (result.Configuration)
                this.setConfiguration(result.Configuration);
            return this;
        });
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            if (yield this.exists())
                return this;
            const result = yield this.region.getClient().send(new client_lambda_1.CreateFunctionCommand({
                FunctionName: this.getName(),
                Role: '',
                Runtime: 'nodejs16.x',
                Description: this.getDescription(),
                Handler: this.getHandler(),
                MemorySize: this.getMemorySize(),
                Publish: true,
                Code: {
                    ZipFile: new Buffer('')
                }
            }));
            return this.setConfiguration(result);
        });
    }
    createUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.region.getClient().send(new client_lambda_1.CreateFunctionUrlConfigCommand({
                FunctionName: this.getName(),
                AuthType: 'NONE',
                Cors: {
                    AllowCredentials: true,
                    AllowHeaders: ['X-CloudFront-Sec-Token', 'Origin', 'Date', 'Keep-Alive'],
                    AllowMethods: ['HEAD', 'GET', 'PUT', 'POST', 'PATCH', 'DELETE'],
                    AllowOrigins: ['https://']
                }
            }));
            yield this.region.getClient().send(new client_lambda_1.AddPermissionCommand({
                FunctionName: this.getName(),
                StatementId: 'PublicAccess',
                FunctionUrlAuthType: 'NONE',
                Principal: '*',
                Action: 'lambda:InvokeFunctionUrl'
            }));
        });
    }
    getName() {
        if (this.configuration)
            return this.configuration.FunctionName;
        return this.name;
    }
    getOriginName() {
        return [this.region.getId(), this.getName()].join('_');
    }
    getDomainName() {
        return '';
    }
    getDescription() {
        if (this.configuration)
            return this.configuration.Description;
        return '';
    }
    setConfiguration(configuration) {
        this.configuration = configuration;
        this.functionExists = true;
        return this;
    }
    getHandler() {
        return this.handler;
    }
    getMemorySize() {
        return this.memorySize;
    }
}
exports.LambdaFunction = LambdaFunction;
