"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LambdaRegion = void 0;
const client_lambda_1 = require("@aws-sdk/client-lambda");
const _1 = require(".");
class LambdaRegion {
    /**
     * @constructor
     */
    constructor(region) {
        this.client = new client_lambda_1.LambdaClient({ region });
        this.region = region;
    }
    /**
     * @func 	listFunctions
     * @desc 	List all Compute functions in the given AWS region
     */
    listFunctions() {
        return __awaiter(this, void 0, void 0, function* () {
            const functions = [];
            let next;
            while (true) {
                const result = yield this.client.send(new client_lambda_1.ListFunctionsCommand({
                    MaxItems: 50,
                    Marker: next // Paginate to next result
                }));
                // Add a function instance for each returned configuration
                if (result.Functions && result.Functions.length > 0)
                    functions.push(...result.Functions.map((configuration) => new _1.LambdaFunction(this, configuration.FunctionName).setConfiguration(configuration)));
                // Paginate through results
                if (result.NextMarker)
                    next = result.NextMarker;
                else
                    break;
            }
            return functions;
        });
    }
    hasFunction(name) {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    getId() {
        return this.region;
    }
    getName() {
        return this.region;
    }
    getClient() {
        return this.client;
    }
}
exports.LambdaRegion = LambdaRegion;
